@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2009 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -54,12 +54,18 @@ typedef enum
 	FLT_O_REGEX	= 1 << 2
 } FilterOldFlag;
 
+static FilterInAddressBookFunc default_addrbook_func = NULL;
+
 static gboolean filter_match_cond	(FilterCond	*cond,
 					 MsgInfo	*msginfo,
 					 GSList		*hlist,
 					 FilterInfo	*fltinfo);
 static gboolean filter_match_header_cond(FilterCond	*cond,
 					 GSList		*hlist);
+static gboolean filter_match_in_addressbook
+					(FilterCond	*cond,
+					 GSList		*hlist,
+					 FilterInfo	*fltinfo);
 
 static void filter_cond_free		(FilterCond	*cond);
 static void filter_action_free		(FilterAction	*action);
@@ -244,12 +250,17 @@ gint filter_action_exec(FilterRule *rule, MsgInfo *msginfo, const gchar *file,
 					fltinfo->actions[action->type] = TRUE;
 				}
 			} else {
-				if (folder_item_add_msg(dest_folder, file,
-							&fltinfo->flags,
-							FALSE) < 0) {
+				MsgFlags save_flags;
+
+				save_flags = msginfo->flags;
+				msginfo->flags = fltinfo->flags;
+				if (folder_item_add_msg_msginfo
+					(dest_folder, msginfo, FALSE) < 0) {
+					msginfo->flags = save_flags;
 					fltinfo->error = FLT_ERROR_ERROR;
 					return -1;
 				}
+				msginfo->flags = save_flags;
 				fltinfo->actions[action->type] = TRUE;
 			}
 
@@ -383,9 +394,17 @@ static gboolean filter_match_cond(FilterCond *cond, MsgInfo *msginfo,
 
 	switch (cond->type) {
 	case FLT_COND_HEADER:
+		if (cond->match_type == FLT_IN_ADDRESSBOOK)
+			return filter_match_in_addressbook(cond, hlist, fltinfo);
+		else
+			return filter_match_header_cond(cond, hlist);
 	case FLT_COND_ANY_HEADER:
-	case FLT_COND_TO_OR_CC:
 		return filter_match_header_cond(cond, hlist);
+	case FLT_COND_TO_OR_CC:
+		if (cond->match_type == FLT_IN_ADDRESSBOOK)
+			return filter_match_in_addressbook(cond, hlist, fltinfo);
+		else
+			return filter_match_header_cond(cond, hlist);
 	case FLT_COND_BODY:
 		matched = procmime_find_string(msginfo, cond->str_value,
 					       cond->match_func);
@@ -395,7 +414,7 @@ static gboolean filter_match_cond(FilterCond *cond, MsgInfo *msginfo,
 		if (!file)
 			return FALSE;
 		cmdline = g_strconcat(cond->str_value, " \"", file, "\"", NULL);
-		ret = execute_command_line(cmdline, FALSE);
+		ret = execute_command_line_async_wait(cmdline);
 		matched = (ret == 0);
 		if (ret == -1)
 			fltinfo->error = FLT_ERROR_EXEC_FAILED;
@@ -485,6 +504,45 @@ static gboolean filter_match_header_cond(FilterCond *cond, GSList *hlist)
 	return matched;
 }
 
+static gboolean filter_match_in_addressbook(FilterCond *cond, GSList *hlist,
+					    FilterInfo *fltinfo)
+{
+	gboolean matched = FALSE;
+	GSList *cur;
+	Header *header;
+
+	if (!default_addrbook_func)
+		return FALSE;
+	if (cond->type != FLT_COND_HEADER && cond->type != FLT_COND_TO_OR_CC)
+		return FALSE;
+
+	for (cur = hlist; cur != NULL; cur = cur->next) {
+		header = (Header *)cur->data;
+
+		if (cond->type == FLT_COND_HEADER) {
+			if (!g_ascii_strcasecmp
+				(header->name, cond->header_name)) {
+				if (default_addrbook_func(header->body))
+					matched = TRUE;
+			}
+		} else if (cond->type == FLT_COND_TO_OR_CC) {
+			if (!g_ascii_strcasecmp(header->name, "To") ||
+			    !g_ascii_strcasecmp(header->name, "Cc")) {
+				if (default_addrbook_func(header->body))
+					matched = TRUE;
+			}
+		}
+
+		if (matched == TRUE)
+			break;
+	}
+
+	if (FLT_IS_NOT_MATCH(cond->match_flag))
+		matched = !matched;
+
+	return matched;
+}
+
 gboolean filter_rule_requires_full_headers(FilterRule *rule)
 {
 	GSList *cur;
@@ -816,7 +874,8 @@ void filter_write_file(GSList *list, const gchar *file)
 		FilterRule *rule = (FilterRule *)cur->data;
 		GSList *cur_cond;
 		GSList *cur_action;
-		gchar match_type[16];
+		gchar match_type[64];
+		gchar nstr[16];
 
 		fputs("    <rule name=\"", pfile->fp);
 		xml_file_put_escape_str(pfile->fp, rule->name);
@@ -836,19 +895,27 @@ void filter_write_file(GSList *list, const gchar *file)
 
 			switch (cond->match_type) {
 			case FLT_CONTAIN:
-				strcpy(match_type,
-				       FLT_IS_NOT_MATCH(cond->match_flag)
-				       ? "not-contain" : "contains");
+				strncpy2(match_type,
+					 FLT_IS_NOT_MATCH(cond->match_flag)
+					 ? "not-contain" : "contains",
+					 sizeof(match_type));
 				break;
 			case FLT_EQUAL:
-				strcpy(match_type,
-				       FLT_IS_NOT_MATCH(cond->match_flag)
-				       ? "is-not" : "is");
+				strncpy2(match_type,
+					 FLT_IS_NOT_MATCH(cond->match_flag)
+					 ? "is-not" : "is", sizeof(match_type));
 				break;
 			case FLT_REGEX:
-				strcpy(match_type,
-				       FLT_IS_NOT_MATCH(cond->match_flag)
-				       ? "not-regex" : "regex");
+				strncpy2(match_type,
+					 FLT_IS_NOT_MATCH(cond->match_flag)
+					 ? "not-regex" : "regex",
+					 sizeof(match_type));
+				break;
+			case FLT_IN_ADDRESSBOOK:
+				strncpy2(match_type,
+					 FLT_IS_NOT_MATCH(cond->match_flag)
+					 ? "not-in-addressbook" : "in-addressbook",
+					 sizeof(match_type));
 				break;
 			default:
 				match_type[0] = '\0';
@@ -885,13 +952,13 @@ void filter_write_file(GSList *list, const gchar *file)
 				NODE_NEW("command-test", cond->str_value);
 				break;
 			case FLT_COND_SIZE_GREATER:
-				NODE_NEW("size", itos(cond->int_value));
+				NODE_NEW("size", itos_buf(nstr, cond->int_value));
 				ADD_ATTR("type",
 					 FLT_IS_NOT_MATCH(cond->match_flag)
 					 ? "lt" : "gt");
 				break;
 			case FLT_COND_AGE_GREATER:
-				NODE_NEW("age", itos(cond->int_value));
+				NODE_NEW("age", itos_buf(nstr, cond->int_value));
 				ADD_ATTR("type",
 					 FLT_IS_NOT_MATCH(cond->match_flag)
 					 ? "lt" : "gt");
@@ -921,7 +988,7 @@ void filter_write_file(GSList *list, const gchar *file)
 					 ? "is-not" : "is");
 				break;
 			case FLT_COND_ACCOUNT:
-				 NODE_NEW("account-id", itos(cond->int_value));
+				 NODE_NEW("account-id", itos_buf(nstr, cond->int_value));
 				 break;
 			default:
 				 break;
@@ -1200,6 +1267,16 @@ FilterRule *filter_read_str(const gchar *str)
 	return rule;
 }
 
+void filter_set_addressbook_func(FilterInAddressBookFunc func)
+{
+	default_addrbook_func = func;
+}
+
+FilterInAddressBookFunc filter_get_addressbook_func(void)
+{
+	return default_addrbook_func;
+}
+
 FilterRule *filter_rule_new(const gchar *name, FilterBoolOp bool_op,
 			    GSList *cond_list, GSList *action_list)
 {
@@ -1247,6 +1324,8 @@ FilterCond *filter_cond_new(FilterCondType type,
 			cond->match_func = str_find_equal;
 		else
 			cond->match_func = str_case_find_equal;
+	} else if (match_type == FLT_IN_ADDRESSBOOK) {
+		cond->match_func = str_case_find_equal;
 	} else {
 		if (FLT_IS_CASE_SENS(match_flag))
 			cond->match_func = str_find;
@@ -1415,6 +1494,11 @@ void filter_rule_match_type_str_to_enum(const gchar *match_type,
 	} else if (!strcmp(match_type, "not-regex")) {
 		*type = FLT_REGEX;
 		*flag = FLT_NOT_MATCH;
+	} else if (!strcmp(match_type, "in-addressbook")) {
+		*type = FLT_IN_ADDRESSBOOK;
+	} else if (!strcmp(match_type, "not-in-addressbook")) {
+		*type = FLT_IN_ADDRESSBOOK;
+		*flag = FLT_NOT_MATCH;
 	} else if (!strcmp(match_type, "gt")) {
 	} else if (!strcmp(match_type, "lt")) {
 		*flag = FLT_NOT_MATCH;
