@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -358,15 +358,15 @@ void procmime_scan_encoding(MimeInfo *mimeinfo, const gchar *encoding)
 	g_free(mimeinfo->encoding);
 
 	mimeinfo->encoding = g_strdup(g_strstrip(buf));
-	if (!g_ascii_strcasecmp(buf, "7bit"))
+	if (!g_ascii_strncasecmp(buf, "7bit", 4))
 		mimeinfo->encoding_type = ENC_7BIT;
-	else if (!g_ascii_strcasecmp(buf, "8bit"))
+	else if (!g_ascii_strncasecmp(buf, "8bit", 4))
 		mimeinfo->encoding_type = ENC_8BIT;
-	else if (!g_ascii_strcasecmp(buf, "quoted-printable"))
+	else if (!g_ascii_strncasecmp(buf, "quoted-printable", 16))
 		mimeinfo->encoding_type = ENC_QUOTED_PRINTABLE;
-	else if (!g_ascii_strcasecmp(buf, "base64"))
+	else if (!g_ascii_strncasecmp(buf, "base64", 6))
 		mimeinfo->encoding_type = ENC_BASE64;
-	else if (!g_ascii_strcasecmp(buf, "x-uuencode"))
+	else if (!g_ascii_strncasecmp(buf, "x-uuencode", 10))
 		mimeinfo->encoding_type = ENC_X_UUENCODE;
 	else
 		mimeinfo->encoding_type = ENC_UNKNOWN;
@@ -464,6 +464,8 @@ static gchar *procmime_convert_value(const gchar *value, const gchar *charset)
 
 static MimeParams *procmime_parse_mime_parameter(const gchar *str)
 {
+	ConvADType ad_type;
+	gchar *tmp_param = NULL;
 	gchar *hvalue;
 	gchar *param, *name, *value;
 	gchar *charset = NULL, *lang = NULL;
@@ -489,6 +491,19 @@ static MimeParams *procmime_parse_mime_parameter(const gchar *str)
 	if (!p)
 		return mparams;
 	++p;
+
+	/* workaround for raw-JIS filename (Eudora etc.) */
+	ad_type = conv_get_autodetect_type();
+	if ((ad_type == C_AD_JAPANESE ||
+	     (ad_type == C_AD_BY_LOCALE && conv_is_ja_locale())) &&
+	    strstr(p, "\033$") != NULL) {
+		CodeConvFunc conv_func;
+		conv_func = conv_get_code_conv_func(NULL, NULL);
+		tmp_param = conv_func(p, NULL);
+		p = tmp_param;
+		debug_print("procmime_parse_mime_parameter(): raw-JIS header body detected: %s\n", str);
+	}
+
 	count = prev_count = -1;
 	cont_name = cont_value = NULL;
 
@@ -662,6 +677,8 @@ static MimeParams *procmime_parse_mime_parameter(const gchar *str)
 	g_free(cont_value);
 	g_free(lang);
 	g_free(charset);
+	if (tmp_param)
+		g_free(tmp_param);
 
 	plist = g_slist_reverse(plist);
 	mparams->plist = plist;
@@ -703,6 +720,8 @@ void procmime_scan_content_type_str(const gchar *content_type,
 		MimeParam *param = (MimeParam *)cur->data;
 		if (charset && !g_ascii_strcasecmp(param->name, "charset")) {
 			*charset = g_strdup(param->value);
+			eliminate_parenthesis(*charset, '(', ')');
+			g_strstrip(*charset);
 			charset = NULL;
 		} else if (name && !g_ascii_strcasecmp(param->name, "name")) {
 			*name = g_strdup(param->value);
@@ -782,9 +801,11 @@ MimeInfo *procmime_scan_mime_header(FILE *fp)
 	}
 
 	if (mimeinfo->mime_type == MIME_APPLICATION_OCTET_STREAM &&
-	    mimeinfo->name) {
+	    (mimeinfo->filename || mimeinfo->name)) {
 		const gchar *type;
-		type = procmime_get_mime_type(mimeinfo->name);
+		type = procmime_get_mime_type
+			(mimeinfo->filename ? mimeinfo->filename
+			 : mimeinfo->name);
 		if (type)
 			mimeinfo->mime_type = procmime_scan_mime_type(type);
 	}
@@ -795,6 +816,37 @@ MimeInfo *procmime_scan_mime_header(FILE *fp)
 	return mimeinfo;
 }
 
+static gint procmime_normalize_lbreak(FILE *infp, FILE *outfp)
+{
+	gchar buf[BUFFSIZE];
+	gint len;
+
+	g_return_val_if_fail(infp != NULL, -1);
+	g_return_val_if_fail(outfp != NULL, -1);
+
+	while (fgets(buf, sizeof(buf), infp) != NULL) {
+		len = strlen(buf);
+		if (len == sizeof(buf) - 1 && buf[len - 1] != '\n') {
+			if (buf[len - 1] == '\r') {
+				ungetc('\r', infp);
+				buf[len - 1] = '\0';
+			}
+			fputs(buf, outfp);
+			continue;
+		}
+#ifdef G_OS_WIN32
+		strretchomp(buf);
+		fputs(buf, outfp);
+		fputs("\r\n", outfp);
+#else
+		strcrchomp(buf);
+		fputs(buf, outfp);
+#endif
+	}
+
+	return 0;
+}
+
 FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 {
 	gchar buf[BUFFSIZE];
@@ -829,6 +881,7 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 
 	if (mimeinfo->encoding_type == ENC_QUOTED_PRINTABLE) {
 		FILE *tmpfp = outfp;
+		gchar prev_empty_line[3] = "";
 
 		if (normalize_lbreak) {
 			tmpfp = my_tmpfile();
@@ -843,9 +896,22 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 		       (!boundary ||
 			!IS_BOUNDARY(buf, boundary, boundary_len))) {
 			gint len;
-			len = qp_decode_line(buf);
-			fwrite(buf, len, 1, tmpfp);
+
+			if (prev_empty_line[0]) {
+				fputs(prev_empty_line, tmpfp);
+				prev_empty_line[0] = '\0';
+			}
+
+			if (buf[0] == '\n' ||
+			    (buf[0] == '\r' && buf[1] == '\n'))
+				strcpy(prev_empty_line, buf);
+			else {
+				len = qp_decode_line(buf);
+				fwrite(buf, len, 1, tmpfp);
+			}
 		}
+		if (!boundary && prev_empty_line[0])
+			fputs(prev_empty_line, tmpfp);
 
 		if (normalize_lbreak) {
 			if (fflush(tmpfp) == EOF) {
@@ -855,16 +921,7 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 				return NULL;
 			}
 			rewind(tmpfp);
-			while (fgets(buf, sizeof(buf), tmpfp) != NULL) {
-#ifdef G_OS_WIN32
-				strretchomp(buf);
-				fputs(buf, outfp);
-				fputs("\r\n", outfp);
-#else
-				strcrchomp(buf);
-				fputs(buf, outfp);
-#endif
-			}
+			procmime_normalize_lbreak(tmpfp, outfp);
 			fclose(tmpfp);
 		}
 	} else if (mimeinfo->encoding_type == ENC_BASE64) {
@@ -904,16 +961,7 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 				return NULL;
 			}
 			rewind(tmpfp);
-			while (fgets(buf, sizeof(buf), tmpfp) != NULL) {
-#ifdef G_OS_WIN32
-				strretchomp(buf);
-				fputs(buf, outfp);
-				fputs("\r\n", outfp);
-#else
-				strcrchomp(buf);
-				fputs(buf, outfp);
-#endif
-			}
+			procmime_normalize_lbreak(tmpfp, outfp);
 			fclose(tmpfp);
 		}
 	} else if (mimeinfo->encoding_type == ENC_X_UUENCODE) {
@@ -938,21 +986,59 @@ FILE *procmime_decode_content(FILE *outfp, FILE *infp, MimeInfo *mimeinfo)
 				flag = TRUE;
 		}
 	} else {
+		gchar prev_empty_line[3] = "";
+		gint len;
+		gboolean cont_line = FALSE;
+
 		while (fgets(buf, sizeof(buf), infp) != NULL &&
 		       (!boundary ||
 			!IS_BOUNDARY(buf, boundary, boundary_len))) {
+			if (prev_empty_line[0]) {
+				fputs(prev_empty_line, outfp);
+				prev_empty_line[0] = '\0';
+			}
+
+			len = strlen(buf);
+			if (len == sizeof(buf) - 1 &&
+			    buf[len - 1] != '\n') {
+				if (buf[len - 1] == '\r') {
+					ungetc('\r', infp);
+					buf[len - 1] = '\0';
+				}
+				fputs(buf, outfp);
+				cont_line = TRUE;
+				continue;
+			}
+
 			if (normalize_lbreak) {
 #ifdef G_OS_WIN32
 				strretchomp(buf);
-				fputs(buf, outfp);
-				fputs("\r\n", outfp);
+				if (!cont_line && buf[0] == '\0')
+					strcpy(prev_empty_line, "\r\n");
+				else {
+					fputs(buf, outfp);
+					fputs("\r\n", outfp);
+				}
 #else
 				strcrchomp(buf);
-				fputs(buf, outfp);
+				if (!cont_line && buf[0] == '\n')
+					strcpy(prev_empty_line, "\n");
+				else
+					fputs(buf, outfp);
 #endif
-			} else
-				fputs(buf, outfp);
+			} else {
+				if (!cont_line &&
+				    (buf[0] == '\n' ||
+				     (buf[0] == '\r' && buf[1] == '\n')))
+					strcpy(prev_empty_line, buf);
+				else
+					fputs(buf, outfp);
+			}
+
+			cont_line = FALSE;
 		}
+		if (!boundary && prev_empty_line[0])
+			fputs(prev_empty_line, outfp);
 	}
 
 	if (fflush(outfp) == EOF)
@@ -1680,7 +1766,7 @@ EncodingType procmime_get_encoding_for_str(const gchar *str)
 
 	total_len = strlen(str);
 
-	for (p = str; *p != '\0'; ++p) {
+	for (p = (const guchar *)str; *p != '\0'; ++p) {
 		if (*p & 0x80)
 			++octet_chars;
 	}
