@@ -26,12 +26,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkeditable.h>
-#include <gtk/gtkclist.h>
-#include <gtk/gtkscrolledwindow.h>
+#include <gtk/gtk.h>
 
 #include <string.h>
 #include <ctype.h>
@@ -179,7 +174,7 @@ static gint add_address(const gchar *name, const gchar *address, const gchar *ni
 	addr_ce = g_new0(completion_entry, 1);
 
 	ae = g_new0(address_entry, 1);
-	ae->name    = g_strdup(name);
+	ae->name    = g_strdup(name ? name : "");
 	ae->address = g_strdup(address);		
 	if ((found = g_list_find_custom(address_list, ae,
 					address_entry_find_func))) {
@@ -276,15 +271,11 @@ gchar *get_address_from_edit(GtkEntry *entry, gint *start_pos)
 	if (g_utf8_strlen(p, -1) == 0)
 		return NULL;
 
-#define IS_VALID_CHAR(x) \
-	(g_ascii_isalnum(x) || (x) == '"' || (x) == '<' || ((guchar)(x) > 0x7f))
-
 	/* now scan back until we hit a valid character */
-	for (; *p && !IS_VALID_CHAR(*p); p = g_utf8_next_char(p))
+	for (; *p && (*p == ',' || g_ascii_isspace(*p));
+	     p = g_utf8_next_char(p))
 		;
 
-#undef IS_VALID_CHAR
-
 	if (g_utf8_strlen(p, -1) == 0)
 		return NULL;
 
@@ -412,7 +403,7 @@ gchar *get_complete_address(gint index)
 				if (!p->name || p->name[0] == '\0')
 					address = g_strdup(p->address);
 				else if (p->name[0] != '"' &&
-					 strpbrk(p->name, ",.[]<>") != NULL)
+					 strpbrk(p->name, "(),.:;<>@[]") != NULL)
 					address = g_strdup_printf
 						("\"%s\" <%s>", p->name, p->address);
 				else
@@ -763,7 +754,7 @@ static gboolean address_completion_entry_key_pressed(GtkEntry    *entry,
 static gboolean address_completion_complete_address_in_entry(GtkEntry *entry,
 							     gboolean  next)
 {
-	gint ncount, cursor_pos;
+	gint ncount = 0, cursor_pos;
 	gchar *address, *new = NULL;
 	gboolean completed = FALSE;
 
@@ -787,8 +778,13 @@ static gboolean address_completion_complete_address_in_entry(GtkEntry *entry,
 	if (new) {
 		/* prevent "change" signal */
 		/* replace_address_in_edit(entry, new, cursor_pos); */
+
+		/* don't complete if entry equals to the completed address */
+		if (ncount == 2 && !strcmp(address, new))
+			completed = FALSE;
+		else
+			completed = TRUE;
 		g_free(new);
-		completed = TRUE;
 	}
 
 	g_free(address);
