@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,45 +25,47 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdk.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkhbbox.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkarrow.h>
-#include <gtk/gtkctree.h>
-#include <gtk/gtkcombo.h>
-#include <gtk/gtkbindings.h>
-#include <gtk/gtkitemfactory.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtktreesortable.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreestore.h>
-#include <gtk/gtkversion.h>
+#include <gtk/gtk.h>
 #include <stdlib.h>
+#include <string.h>
 #include <stdarg.h>
 
+#ifdef G_OS_WIN32
+#  include <pango/pangowin32.h>
+#endif
+
 #include "gtkutils.h"
 #include "utils.h"
 #include "codeconv.h"
 #include "menu.h"
 
-gboolean gtkut_get_font_size(GtkWidget *widget, gint *width, gint *height)
+gboolean gtkut_get_str_size(GtkWidget *widget, const gchar *str,
+			    gint *width, gint *height)
 {
 	PangoLayout *layout;
-	const gchar *str = "Abcdef";
 
 	g_return_val_if_fail(GTK_IS_WIDGET(widget), FALSE);
 
 	layout = gtk_widget_create_pango_layout(widget, str);
 	g_return_val_if_fail(layout, FALSE);
 	pango_layout_get_pixel_size(layout, width, height);
-	if (width)
-		*width = *width / g_utf8_strlen(str, -1);
 	g_object_unref(layout);
 
 	return TRUE;
 }
 
+gboolean gtkut_get_font_size(GtkWidget *widget, gint *width, gint *height)
+{
+	const gchar *str = "Abcdef";
+	gboolean ret;
+
+	ret = gtkut_get_str_size(widget, str, width, height);
+	if (ret && width)
+		*width = *width / g_utf8_strlen(str, -1);
+
+	return ret;
+}
+
 PangoFontDescription *gtkut_get_default_font_desc(void)
 {
 	static PangoFontDescription *font_desc = NULL;
@@ -96,6 +98,32 @@ void gtkut_widget_set_small_font_size(GtkWidget *widget)
 	pango_font_description_free(font_desc);
 }
 
+gboolean gtkut_font_can_load(const gchar *str)
+{
+#ifdef G_OS_WIN32
+	PangoFontDescription *desc;
+	PangoContext *context;
+	PangoFont *font;
+	gboolean can_load = FALSE;
+
+	desc = pango_font_description_from_string(str);
+	if (desc) {
+		context = pango_win32_get_context();
+		font = pango_context_load_font(context, desc);
+		if (font) {
+			can_load = TRUE;
+			g_object_unref(font);
+		}
+		g_object_unref(context);
+		pango_font_description_free(desc);
+	}
+
+	return can_load;
+#else
+	return FALSE;
+#endif
+}
+
 void gtkut_convert_int_to_gdk_color(gint rgbvalue, GdkColor *color)
 {
 	g_return_if_fail(color != NULL);
@@ -262,10 +290,12 @@ ComboButton *gtkut_combo_button_create(GtkWidget *button,
 
 	g_signal_connect(G_OBJECT(combo->button), "size_request",
 			 G_CALLBACK(combo_button_size_request), combo);
+#if 0
 	g_signal_connect(G_OBJECT(combo->button), "enter",
 			 G_CALLBACK(combo_button_enter), combo);
 	g_signal_connect(G_OBJECT(combo->button), "leave",
 			 G_CALLBACK(combo_button_leave), combo);
+#endif
 	g_signal_connect(G_OBJECT(combo->arrow), "enter",
 			 G_CALLBACK(combo_button_enter), combo);
 	g_signal_connect(G_OBJECT(combo->arrow), "leave",
@@ -298,6 +328,25 @@ void gtkut_clist_set_focus_row(GtkCList *clist, gint row)
 	GTKUT_CTREE_REFRESH(clist);
 }
 
+#ifdef G_OS_WIN32
+static void vadjustment_changed(GtkAdjustment *adj, gpointer data)
+{
+	GtkWidget *widget = GTK_WIDGET(data);
+
+	gtk_widget_queue_draw(widget);
+}
+#endif
+
+void gtkut_clist_set_redraw(GtkCList *clist)
+{
+#ifdef G_OS_WIN32
+	if (clist->vadjustment) {
+		g_signal_connect(G_OBJECT(clist->vadjustment), "changed",
+				 G_CALLBACK(vadjustment_changed), clist);
+	}
+#endif
+}
+
 gboolean gtkut_tree_model_next(GtkTreeModel *model, GtkTreeIter *iter)
 {
 	GtkTreeIter iter_, parent;
@@ -689,6 +738,19 @@ void gtkut_editable_disable_im(GtkEditable *editable)
 #endif
 }
 
+void gtkut_entry_strip_text(GtkEntry *entry)
+{
+	gchar *text;
+	gint len;
+
+	text = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
+	len = strlen(text);
+	g_strstrip(text);
+	if (len > strlen(text))
+		gtk_entry_set_text(entry, text);
+	g_free(text);
+}
+
 void gtkut_container_remove(GtkContainer *container, GtkWidget *widget)
 {
 	gtk_container_remove(container, widget);
@@ -878,7 +940,12 @@ void gtkut_window_popup(GtkWidget *window)
 	if (new_x != x || new_y != y)
 		gdk_window_move(window->window, new_x, new_y);
 
+	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(window), FALSE);
 	gtk_window_present(GTK_WINDOW(window));
+#ifdef G_OS_WIN32
+	/* ensure that the window is displayed at the top */
+	gdk_window_show(window->window);
+#endif
 }
 
 gboolean gtkut_window_modal_exist(void)
