@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2010 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,6 +54,8 @@
 #include "procheader.h"
 #include "summaryview.h"
 #include "menu.h"
+#include "compose.h"
+#include "printing.h"
 #include "filesel.h"
 #include "alertpanel.h"
 #include "inputdialog.h"
@@ -120,6 +122,10 @@ static void mimeview_open_with		(MimeView	*mimeview);
 static void mimeview_view_file		(const gchar	*filename,
 					 MimeInfo	*partinfo,
 					 const gchar	*cmdline);
+
+static void mimeview_reply		(MimeView	*mimeview,
+					 guint		 action);
+
 #if USE_GPGME
 static void mimeview_check_signature	(MimeView	*mimeview);
 #endif
@@ -131,9 +137,18 @@ static GtkItemFactoryEntry mimeview_popup_entries[] =
 	{N_("/_Display as text"), NULL, mimeview_display_as_text, 0, NULL},
 	{N_("/_Save as..."),	  NULL, mimeview_save_as,	  0, NULL},
 	{N_("/Save _all..."),	  NULL, mimeview_save_all,	  0, NULL},
-	{N_("/_Print..."),	  NULL, mimeview_print,		  0, NULL}
+	{N_("/_Print..."),	  NULL, mimeview_print,		  0, NULL},
+	{N_("/---"),		  NULL, NULL,			  0, "<Separator>"},
+	{N_("/_Reply"),		  NULL, NULL,			  0, "<Branch>"},
+	{N_("/_Reply/_Reply"),	  NULL, mimeview_reply,		  COMPOSE_REPLY, NULL},
+	{N_("/_Reply/Reply to _all"),
+				  NULL, mimeview_reply,		  COMPOSE_REPLY_TO_ALL, NULL},
+	{N_("/_Reply/Reply to _sender"),
+				  NULL, mimeview_reply,		  COMPOSE_REPLY_TO_SENDER, NULL},
+	{N_("/_Reply/Reply to mailing _list"),
+				  NULL, mimeview_reply,		  COMPOSE_REPLY_TO_LIST, NULL},
 #if USE_GPGME
-        ,
+	{N_("/---"),		  NULL, NULL,			  0, "<Separator>"},
         {N_("/_Check signature"), NULL, mimeview_check_signature, 0, NULL}
 #endif
 };
@@ -158,6 +173,9 @@ MimeView *mimeview_create(void)
 	GtkWidget *popupmenu;
 	GtkItemFactory *popupfactory;
 	gint n_entries;
+	GtkWidget *reply_separator;
+	GtkWidget *reply_menuitem;
+	GList *child;
 
 	debug_print(_("Creating MIME view...\n"));
 	mimeview = g_new0(MimeView, 1);
@@ -240,6 +258,11 @@ MimeView *mimeview_create(void)
 	popupmenu = menu_create_items(mimeview_popup_entries, n_entries,
 				      "<MimeView>", &popupfactory, mimeview);
 
+	reply_menuitem = gtk_item_factory_get_item(popupfactory, "/Reply");
+	child = g_list_find(GTK_MENU_SHELL(popupmenu)->children,
+			    reply_menuitem);
+	reply_separator = GTK_WIDGET(child->prev->data);
+
 	mimeview->paned        = paned;
 	mimeview->scrolledwin  = scrolledwin;
 	mimeview->treeview     = treeview;
@@ -248,6 +271,8 @@ MimeView *mimeview_create(void)
 	mimeview->mime_vbox    = mime_vbox;
 	mimeview->popupmenu    = popupmenu;
 	mimeview->popupfactory = popupfactory;
+	mimeview->reply_separator = reply_separator;
+	mimeview->reply_menuitem  = reply_menuitem;
 	mimeview->type         = -1;
 
 	return mimeview;
@@ -271,6 +296,7 @@ static gboolean mimeview_is_signed(MimeView *mimeview)
 
         if (!mimeview) return FALSE;
 	if (!mimeview->opened) return FALSE;
+	if (!rfc2015_is_available()) return FALSE;
 
         debug_print("mimeview_is_signed: open\n" );
 
@@ -296,6 +322,8 @@ static void set_unchecked_signature(MimeInfo *mimeinfo)
 {
 	MimeInfo **signedinfo;
 
+	if (!rfc2015_is_available()) return;
+
 	signedinfo = rfc2015_find_signature(mimeinfo);
 	if (signedinfo == NULL) return;
 
@@ -328,7 +356,7 @@ void mimeview_show_message(MimeView *mimeview, MimeInfo *mimeinfo,
 	mimeview->file = g_strdup(file);
 
 #if USE_GPGME
-	if (prefs_common.auto_check_signatures) {
+	if (rfc2015_is_available() && prefs_common.auto_check_signatures) {
 		FILE *fp;
 
 		if ((fp = g_fopen(file, "rb")) == NULL) {
@@ -694,10 +722,13 @@ static void mimeview_show_signature_part(MimeView *mimeview,
 	vbbox = gtk_vbutton_box_new();
 	gtk_box_set_spacing(GTK_BOX(vbbox), 5);
 
-	button = gtk_button_new_with_mnemonic(_("_Check signature"));
-	gtk_container_add(GTK_CONTAINER(vbbox), button);
-	g_signal_connect(button, "clicked",
-			 G_CALLBACK(check_signature_button_clicked), mimeview);
+	if (rfc2015_is_available()) {
+		button = gtk_button_new_with_mnemonic(_("_Check signature"));
+		gtk_container_add(GTK_CONTAINER(vbbox), button);
+		g_signal_connect(button, "clicked",
+				 G_CALLBACK(check_signature_button_clicked),
+				 mimeview);
+	}
 
 	gtk_widget_show_all(vbbox);
 
@@ -854,6 +885,14 @@ static gint mimeview_button_pressed(GtkWidget *widget, GdkEventButton *event,
 		else
 			menu_set_sensitive(mimeview->popupfactory,
 					   "/Print...", FALSE);
+
+		if (partinfo && partinfo->mime_type == MIME_MESSAGE_RFC822) {
+			gtk_widget_show(mimeview->reply_separator);
+			gtk_widget_show(mimeview->reply_menuitem);
+		} else {
+			gtk_widget_hide(mimeview->reply_separator);
+			gtk_widget_hide(mimeview->reply_menuitem);
+		}
 #if USE_GPGME
 		menu_set_sensitive(mimeview->popupfactory,
 				   "/Check signature",
@@ -1221,6 +1260,47 @@ static void mimeview_view_file(const gchar *filename, MimeInfo *partinfo,
 		g_warning("MIME viewer command line is invalid: '%s'", cmd);
 }
 
+static void mimeview_reply(MimeView *mimeview, guint action)
+{
+	MimeInfo *partinfo;
+	gchar *filename;
+	MsgInfo *msginfo;
+	MsgFlags flags = {0, 0};
+	ComposeMode mode = action;
+
+	if (!mimeview->opened) return;
+	if (!mimeview->file) return;
+
+	partinfo = mimeview_get_selected_part(mimeview);
+	g_return_if_fail(partinfo != NULL);
+
+	if (partinfo->mime_type != MIME_MESSAGE_RFC822)
+		return;
+
+	filename = procmime_get_tmp_file_name(partinfo);
+	if (procmime_get_part(filename, mimeview->file, partinfo) < 0) {
+		alertpanel_error
+			(_("Can't save the part of multipart message."));
+			g_free(filename);
+			return;
+	}
+
+	msginfo = procheader_parse_file(filename, flags, TRUE);
+	msginfo->file_path = filename;
+	filename = NULL;
+	if (prefs_common.reply_with_quote)
+		mode |= COMPOSE_WITH_QUOTE;
+
+	if (mimeview->messageview->mainwin)
+		compose_reply(msginfo, mimeview->messageview->mainwin->summaryview->folder_item,
+			      mode, NULL);
+	else
+		compose_reply(msginfo, NULL, mode, NULL);
+
+	g_unlink(msginfo->file_path);
+	procmsg_msginfo_free(msginfo);
+}
+
 #if USE_GPGME
 static gboolean update_node_name_func(GtkTreeModel *model, GtkTreePath *path,
 				      GtkTreeIter *iter, gpointer data)
@@ -1267,6 +1347,9 @@ static void mimeview_check_signature(MimeView *mimeview)
 
 	g_return_if_fail (mimeview_is_signed(mimeview));
 
+	if (!rfc2015_is_available())
+		return;
+
 	mimeinfo = mimeview_get_selected_part(mimeview);
 	g_return_if_fail(mimeinfo != NULL);
 	g_return_if_fail(mimeview->file != NULL);
