@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2009 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -105,6 +105,8 @@ static struct QuerySearchWindow {
 
 	gboolean exclude_trash;
 
+	gint n_found;
+
 	gboolean on_search;
 	gboolean cancelled;
 } search_window;
@@ -496,6 +498,7 @@ static FilterRule *query_search_dialog_to_rule(const gchar *name,
 static void query_search_query(void)
 {
 	FolderItem *item;
+	gchar *msg;
 
 	if (search_window.on_search)
 		return;
@@ -518,6 +521,7 @@ static void query_search_query(void)
 	} else
 		search_window.exclude_trash = FALSE;
 
+	search_window.n_found = 0;
 	search_window.cancelled = FALSE;
 
 	gtk_widget_set_sensitive(search_window.clear_btn, FALSE);
@@ -539,70 +543,112 @@ static void query_search_query(void)
 	gtk_widget_set_sensitive(search_window.clear_btn, TRUE);
 	gtk_button_set_label(GTK_BUTTON(search_window.search_btn),
 			     GTK_STOCK_FIND);
-	gtk_label_set_text(GTK_LABEL(search_window.status_label), _("Done."));
+	if (search_window.n_found == 0)
+		msg = g_strdup_printf(_("Message not found."));
+	else if (search_window.n_found == 1)
+		msg = g_strdup_printf(_("1 message found."));
+	else
+		msg = g_strdup_printf(_("%d messages found."),
+				      search_window.n_found);
+	gtk_label_set_text(GTK_LABEL(search_window.status_label), msg);
+	g_free(msg);
 	statusbar_pop_all();
 
 	if (search_window.cancelled)
 		debug_print("* query search cancelled.\n");
 	debug_print("query search finished.\n");
 
+	search_window.n_found = 0;
 	search_window.on_search = FALSE;
 	search_window.cancelled = FALSE;
 }
 
-static void query_search_folder(FolderItem *item)
+typedef struct _QueryData
 {
-	gchar *folder_name, *str;
+	FolderItem *item;
+	gchar *folder_name;
+	gint count;
+	gint total;
+	gint flag;
+	GTimeVal tv_prev;
 	GSList *mlist;
-	FilterInfo fltinfo;
-	GSList *cur;
-	gint count = 1, total;
-	GTimeVal tv_prev, tv_cur;
+#if USE_THREADS
+	GAsyncQueue *queue;
+	guint timer_tag;
+#endif
+} QueryData;
 
-	if (!item->path || item->stype == F_VIRTUAL)
-		return;
+static void query_search_folder_show_progress(const gchar *name, gint count,
+					      gint total)
+{
+	gchar *str;
 
-	folder_name = g_path_get_basename(item->path);
-	str = g_strdup_printf(_("Searching %s ..."), folder_name);
+	str = g_strdup_printf(_("Searching %s (%d / %d)..."),
+			      name, count, total);
 	gtk_label_set_text(GTK_LABEL(search_window.status_label), str);
 	g_free(str);
-	g_get_current_time(&tv_prev);
+#ifndef USE_THREADS
 	ui_update();
+#endif
+}
 
-	if (search_window.cancelled) {
-		g_free(folder_name);
-		return;
-	}
+#if USE_THREADS
+static gboolean query_search_progress_func(gpointer data)
+{
+	QueryData *qdata = (QueryData *)data;
+	MsgInfo *msginfo;
 
-	if (item->opened)
-		summary_write_cache(main_window_get()->summaryview);
+	gdk_threads_enter();
+	query_search_folder_show_progress(qdata->folder_name,
+					  g_atomic_int_get(&qdata->count),
+					  qdata->total);
+	while ((msginfo = g_async_queue_try_pop(qdata->queue)))
+		query_search_append_msg(msginfo);
+	gdk_threads_leave();
 
-	mlist = folder_item_get_msg_list(item, TRUE);
-	total = g_slist_length(mlist);
+	return TRUE;
+}
+#endif
+
+static gpointer query_search_folder_func(gpointer data)
+{
+	QueryData *qdata = (QueryData *)data;
+	GSList *mlist, *cur;
+	FilterInfo fltinfo;
+	GTimeVal tv_cur;
+
+	debug_print("query_search_folder_func start\n");
+
+#if USE_THREADS
+	g_async_queue_ref(qdata->queue);
+#endif
+
+	mlist = qdata->mlist;
 
 	memset(&fltinfo, 0, sizeof(FilterInfo));
 
 	debug_print("requires_full_headers: %d\n",
 		    search_window.requires_full_headers);
-	debug_print("start query search: %s\n", item->path ? item->path : "");
+	debug_print("start query search: %s\n",
+		    qdata->item->path ? qdata->item->path : "");
 
 	for (cur = mlist; cur != NULL; cur = cur->next) {
 		MsgInfo *msginfo = (MsgInfo *)cur->data;
 		GSList *hlist;
 
+		g_atomic_int_add(&qdata->count, 1);
+
 		g_get_current_time(&tv_cur);
-		if (tv_cur.tv_sec > tv_prev.tv_sec ||
-		    tv_cur.tv_usec - tv_prev.tv_usec >
+		if ((tv_cur.tv_sec - qdata->tv_prev.tv_sec) * G_USEC_PER_SEC +
+		    tv_cur.tv_usec - qdata->tv_prev.tv_usec >
 		    PROGRESS_UPDATE_INTERVAL * 1000) {
-			str = g_strdup_printf(_("Searching %s (%d / %d)..."),
-					      folder_name, count, total);
-			gtk_label_set_text
-				(GTK_LABEL(search_window.status_label), str);
-			g_free(str);
-			ui_update();
-			tv_prev = tv_cur;
+#ifndef USE_THREADS
+			query_search_folder_show_progress(qdata->folder_name,
+							  qdata->count,
+							  qdata->total);
+#endif
+			qdata->tv_prev = tv_cur;
 		}
-		++count;
 
 		if (search_window.cancelled)
 			break;
@@ -622,15 +668,90 @@ static void query_search_folder(FolderItem *item)
 
 		if (filter_match_rule(search_window.rule, msginfo, hlist,
 				      &fltinfo)) {
+#if USE_THREADS
+			g_async_queue_push(qdata->queue, msginfo);
+#else
 			query_search_append_msg(msginfo);
+#endif
 			cur->data = NULL;
+			search_window.n_found++;
 		}
 
 		procheader_header_list_destroy(hlist);
 	}
 
-	procmsg_msg_list_free(mlist);
-	g_free(folder_name);
+#if USE_THREADS
+	g_async_queue_unref(qdata->queue);
+#endif
+
+	g_atomic_int_set(&qdata->flag, 1);
+	g_main_context_wakeup(NULL);
+
+	debug_print("query_search_folder_func end\n");
+
+	return GINT_TO_POINTER(0);
+}
+
+static void query_search_folder(FolderItem *item)
+{
+	gchar *str;
+	QueryData data = {item};
+#if USE_THREADS
+	GThread *thread;
+	MsgInfo *msginfo;
+#endif
+
+	if (!item->path || item->stype == F_VIRTUAL)
+		return;
+
+	data.folder_name = g_path_get_basename(item->path);
+	str = g_strdup_printf(_("Searching %s ..."), data.folder_name);
+	gtk_label_set_text(GTK_LABEL(search_window.status_label), str);
+	g_free(str);
+	g_get_current_time(&data.tv_prev);
+#ifndef USE_THREADS
+	ui_update();
+#endif
+
+	if (search_window.cancelled) {
+		g_free(data.folder_name);
+		return;
+	}
+
+	if (item->opened)
+		summary_write_cache(main_window_get()->summaryview);
+
+	procmsg_set_auto_decrypt_message(FALSE);
+
+	data.mlist = folder_item_get_msg_list(item, TRUE);
+	data.total = g_slist_length(data.mlist);
+
+#if USE_THREADS
+	data.queue = g_async_queue_new();
+	data.timer_tag = g_timeout_add(PROGRESS_UPDATE_INTERVAL,
+				       query_search_progress_func, &data);
+	thread = g_thread_create(query_search_folder_func, &data, TRUE, NULL);
+
+	debug_print("query_search_folder: thread started\n");
+	while (g_atomic_int_get(&data.flag) == 0)
+		gtk_main_iteration();
+	log_window_flush();
+
+	while ((msginfo = g_async_queue_try_pop(data.queue)))
+		query_search_append_msg(msginfo);
+
+	g_source_remove(data.timer_tag);
+	g_thread_join(thread);
+	debug_print("query_search_folder: thread exited\n");
+
+	g_async_queue_unref(data.queue);
+#else /* !USE_THREADS */
+	query_search_folder_func(&data);
+#endif
+
+	procmsg_msg_list_free(data.mlist);
+	procmsg_set_auto_decrypt_message(TRUE);
+	g_free(data.folder_name);
 }
 
 static gboolean query_search_recursive_func(GNode *node, gpointer data)
@@ -731,6 +852,7 @@ static void row_activated(GtkTreeView *treeview, GtkTreePath *path,
 				       msginfo)) {
 		msgview = messageview_create_with_new_window();
 		messageview_show(msgview, msginfo, FALSE);
+		statusbar_pop_all();
 	}
 }
 
@@ -973,7 +1095,7 @@ static FolderItem *query_search_create_vfolder(FolderItem *parent,
 	}
 
 	if (parent->path)
-		path = g_strconcat(parent->path, G_DIR_SEPARATOR_S, name, NULL);
+		path = g_strconcat(parent->path, "/", name, NULL);
 	else
 		path = g_strdup(name);
 
