@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2001 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2006 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,9 +27,6 @@
 #include <glib.h>
 #include <gtk/gtktextview.h>
 
-#include <string.h> /* for strlen */
-#include <stdlib.h> /* for mbstowcs */
-
 #include "undo.h"
 #include "utils.h"
 #include "prefs_common.h"
@@ -52,7 +49,7 @@ static gint undo_merge		(GList		*list,
 				 guint		 start_pos,
 				 guint		 end_pos,
 				 gint		 action,
-				 const guchar	*text);
+				 const gchar	*text);
 static void undo_add		(const gchar	*text,
 				 gint		 start_pos,
 				 gint		 end_pos,
@@ -202,9 +199,9 @@ static void undo_check_size(UndoMain *undostruct)
  * Return Value: TRUE is merge was sucessful, FALSE otherwise
  **/
 static gint undo_merge(GList *list, guint start_pos, guint end_pos,
-		       gint action, const guchar *text) 
+		       gint action, const gchar *text) 
 {
-	guchar *temp_string;
+	gchar *temp_string;
 	UndoInfo *last_undo;
 
 	/* This are the cases in which we will NOT merge :
@@ -426,6 +423,8 @@ void undo_undo(UndoMain *undostruct)
 		break;
 	}
 
+	gtk_widget_queue_draw(GTK_WIDGET(textview));
+
 	undostruct->change_state_func(undostruct,
 				      UNDO_STATE_UNCHANGED, UNDO_STATE_TRUE,
 				      undostruct->change_state_data);
@@ -516,6 +515,8 @@ void undo_redo(UndoMain *undostruct)
 		break;
 	}
 
+	gtk_widget_queue_draw(GTK_WIDGET(textview));
+
 	undostruct->change_state_func(undostruct,
 				      UNDO_STATE_TRUE, UNDO_STATE_UNCHANGED, 
 				      undostruct->change_state_data);
